﻿-- TESTING
-- local root = 'er-testing/rso-config-validator/'

-- import all configs from rso 3.7.6 for testing
-- local example_configs = require (root .. 'example_configs/__init__')(root)


--print all words seen in example configs
-- local words = {}
-- for name,config in pairs(example_configs) do
  -- for word,value in pairs(config) do
    -- words[word] = value
    -- end
  -- end
-- print(serpent.block(words))
-- for k in pairs(words) do print(k) end
    
-- ##################### END OF TESTING #####################

-- ROOT PATH OF THIS FILE
local root           = 'config-validator/'
local types          = require (root..'types')
local optionals      = require (root..'optionals')
local specifications = require (root..'specifications') (types)
local SKIP_ENTITY_CHECK  = true --skips entity name test so you don't have to have mods installed

--enables the # operator for all tables
-- local function L(tbl)
  -- return setmetatable(tbl, {
    -- __len = function(self)
      -- local l=0
      -- for _,_ in pairs(self) do
        -- l = l+1 
        -- end
      -- return l
      -- end })
  -- end
  
local function concat (...)
  local tbl = {...}
  local str = ''
  for _,v in pairs(tbl) do
    str = str..'.'..tostring(v)
    end
  return str
  end
  
local function validate_optionals(config,optionals,problems)
  local missing = {}
  for name,bool in pairs(optionals) do
    if (bool ~= true) and (config[name] == nil) then
      table.insert(missing,name)
      end
    end
  for _,name in pairs(missing) do
    table.insert(problems,{name,'is required but missing.'})
    end
  return problems
  end
  
-- local function validate_superflous(config,problems)
  -- local superflous = {}
  -- for name,_ in pairs(config) do
    -- if not specifications[name] then
      -- table.insert(superflous,name)
      -- end
    -- end
  -- for _,name in pairs(superflous) do
    -- table.insert(problems,{name,'is not a valid config key.'})
    -- end
  -- return problems
  -- end
  
local function is_integer(x)
  if type(x) ~= 'number' then return 'is not a number.' end
  local x = tonumber(x)
  if x ~= math.floor(x) then return 'is not an integer.' end
  -- return true
  end
  
local function is_decimal(x)
  if type(x) ~= 'number' then return 'is not a number.' end
  -- return true
  end
  
local function is_string(x)
  if type(x) ~= 'string' then return 'is not a string.' end
  -- return true
  end
  
local function is_boolean(x)
  if type(x) ~= 'boolean' then return 'is not a booealn.' end
  end
  
local function is_table(x)
  if type(x) ~= 'table' then return 'is not a table.' end
  end
  
local function is_entity(x)
  if SKIP_ENTITY_CHECK then return end --DEBUG: allow this even if mod is not installed
  if not game.entity_prototypes[x] then return 'is not an entity.' end
  -- return true
  end
  
local function is_resource(x)
  if SKIP_ENTITY_CHECK then return end --DEBUG: allow this even if mod is not installed
  if not game.entity_prototypes[x] then return 'is not an entity.' end
  if game.entity_prototypes[x].type ~= 'resource' then return 'is not a resource.' end
  -- return true
  end

  
local function validate_node(name,value,s,problems)
  -- value: a single value from a config table, i.e. one entity, i.e. "iron-ore={richness=10}"
  -- s   : the specifications for the node
  -- problems: the table containing all problems found so far
  
  -- print(serpent.block{name,value,s,problems})
  
  local function report(...)
    --a report must be a table {varname,'error_message',reference_value}
    --reference_value is optional, but reports without error_message will be ignored
    local r = {...}
    if r[1] and r[2] then table.insert(problems,r) end
    end
  if     s.type == 'integer'              then
    report(name,is_integer(value))
    if s.min and (value < s.min) then report(name,'is too small.',s.min) end
    if s.max and (value > s.max) then report(name,'is too large.',s.max) end    
    if s.allowed_values and not s.allowed_values[value] then
      report(name,'has a wrong value', s.allowed_values) end     
    
  elseif s.type == 'decimal'              then
    report(name,is_decimal(value))
    if s.min and (value < s.min) then report(name,'is too small.',s.min) end
    if s.max and (value > s.max) then report(name,'is too large.',s.max) end
    
  elseif s.type == 'string'               then
    report(name,is_string(value))
    if s.allowed_values and not s.allowed_values[value] then
      report(name,'has a wrong value', s.allowed_values) end
      
  elseif s.type == 'boolean'              then
    report(name,is_boolean(value))
    
  elseif s.type == 'generic_table'        then
    report(name,is_table(value))
    for k,v in pairs(value) do
      local s2 = s.values[k]
      if s2 then --verify sub nodes only if there is a spec for them!
        validate_node(concat(name,k),v,s2,problems)
        -- validate_node(name..'.'..tostring(k),v,s2,problems)
        -- validate_node(table.concat({name,tostring(k)},'.'),v,s2,problems)
      else
        report(name,'has unnecessary sub-keys.',serpent.line({[k]=v}))
        end
      end
    if (value.min and value.max) and (value.min > value.max) then
      report(name,'min must be smaller or equal to max.')
      end
    
  elseif s.type == 'multi_resource_table' then
    for res_name,count in pairs(value) do
      report(concat(name,res_name),is_entity(res_name))
      validate_node(concat(name,res_name),count,s.count_spec,problems)
      end
    
  elseif s.type == 'resource_name'        then
    report(value,is_resource(value),name)
      
  elseif s.type == 'entity_name'          then
    report(value,is_entity(value),name)
    
  elseif s.type == 'sub_spawns_table'     then
    --TODO.....
    
  elseif s.type == 'table_or_number'      then
    if     type(value) == 'table' then
      validate_node(name,value,s.table ,problems)
    elseif type(value) == 'number' then
      validate_node(name,value,s.number,problems)
    else
      report(name,'is of the wrong type')
      end

  elseif s.type == 'unverified'           then
    -- report(name,'is of type unverified. Please update the spec.')
  else
    error('\n\nRSO config specification error, '..s.type..' is not a valid type.\n')
    end
    
  end
  
local function create_error_message_from_problems(config_name,config,problems)
  if #problems > 0 then
    local err = {
      '\n\nRSO found an invalid configuration:\n\n'..
      config_name..' = '..serpent.line(config)..
      '\n\nThe following problems have been found:\n'}
    for _,report in pairs(problems) do
      local e = '\nValue: "'..report[1]..'" '..report[2]
      if report[3] then e = e..' (Condition was: "' ..serpent.line(report[3])..'".)' end
      table.insert(err,e)
      end 
    return table.concat(err) ..'\n' --compress into a single string
    end
  end

  
local function get_validated_entity_config(config_name,config)
  --inject the entity name into the table
  if (config.type == 'enemy-base') or (config.type == 'resource-ore') then
    config.resource_name = resource_name --verifies if this is an existing resource prototype
    end

  --check if all mandatory values exist
  --inizialize problems table
  local problems = validate_optionals(config,optionals[config.type],{})
  
  --parse each value
  for name,value in pairs(config) do
    local spec = specifications[name]
    if spec then
      validate_node(name,value,spec,problems)
    else
      --catch misspelled key names
      table.insert(problems,{name,'is not a valid config key.'})
      end
    end
  -- print(serpent.block(specifications))
  -- print(serpent.block(config))
  -- print(serpent.block(problems))
  
  if #problems > 0 then
    local err = create_error_message_from_problems(config_name,config,problems)
    return {valid = false, problems = problems, error_message = err}
  else
    return {valid = true , config = config }
    end
    
  end
  
return get_validated_entity_config
  

  

  
  
  


  
  
  
  
  
  
  
  