--[[
  
  TYPES:
    The types table defines generic variable types to make writing 
    the specification less repetitive.
    
  MIN/MAX:
    If any table has a value pair (min/max) they
    are automatically compared to ensure max >= min.
    
  MULTI-RESOURCE-TABLE:
    The special type "multi_resource_table" automatically verifies that
    a resource with that name exists in entity_prototypes.
    
  OPTIONAL/REQUIRED:
    All type functions default to optional=false (i.e. mandatory), and take
    "true" as a final argument to make the value optional.
    
  
  ]]

-- SPECIFICATION
local function specifications (types)
  return {

    type = types.string_with_allowed_values(
      {'resource-liquid','resource-ore','entity'}
      ),
      
    -- entity_name = types.entity_name(),
    resource_name = types.resource_name(),
      
    absolute_probability = types.decimal_with_min_max(0,10000),
    allotment = types.integer_with_min_max(0,10000),
    along_resource_probability = types.decimal_with_min_max(0,10000),
    autoplace_name = types.resource_name(),

    clear_range = types.unverified(), --i don't know what this is
    
    force = types.string_with_allowed_values(
      {'enemy','player','neutral'}
      ),
      
    max_probability_distance_factor = types.decimal_with_min_max(0,10000),
    -- min_amount     = types.integer_with_min_max(0,1e10),
    min_amount     = types.integer_with_min_max(0,1e10),
    min_size       = types.integer_with_min_max(0,10000),
    minimum_amount = types.integer_with_min_max(0,1e10),
    multi_resource = types.multi_resource_table(types.integer_with_min_max(0,100)),
    multi_resource_chance = types.decimal_with_min_max(0,1),
    
    probability_distance_factor = types.decimal_with_min_max(0,10),
    
    -- condition function must return sub-table key (i.e. type(richness)=='table')
    -- note that functions can not be transferred over remote.call
    -- thus we need a global scope function name string
    -- this means any conditional function must be implemented on RSO side.
    -- richness = types.integer_with_min_max(0,1e10),
    richness = types.table_or_number({
      -- condition = 'type', 
      number = types.integer_with_min_max(0,1e10),
      table  = types.generic_table({
        min = types.integer_with_min_max(0,1e10),
        max = types.integer_with_min_max(0,1e10),
        }) }),

    size = types.generic_table({
      min = types.integer_with_min_max(1,10000),
      max = types.integer_with_min_max(1,10000),
      }),
    size_per_region_factor = types.decimal_with_min_max(0,1),
    spawns_per_region = types.generic_table({
      min = types.integer_with_min_max(1,10000),
      max = types.integer_with_min_max(1,10000),
      }),
    starting = types.generic_table({
      probability = types.decimal_with_min_max(0,1),
      richness    = types.decimal_with_min_max(0,1e10),
      size        = types.integer_with_min_max(1,10000),
      }),
    
    sub_spawn_distance_factor = types.decimal_with_min_max(0,10),
    sub_spawn_max_distance_factor = types.integer_with_min_max(1,10000),
    sub_spawn_probability = types.decimal_with_min_max(0,1),
    sub_spawn_size = types.generic_table({
      min = types.integer_with_min_max(1,10000),
      max = types.integer_with_min_max(1,10000),
      }),
    sub_spawns = types.sub_spawns_table({ --entity name is always checked
      allotment = types.integer_with_min_max(1,10000),
      allotment_distance_factor = types.decimal_with_min_max(0,1),
      clear_range = types.unverified(),  --i don't know what this is
      min_distance = types.integer_with_min_max(1,10000),
      }),
    
    useOreScaling = types.boolean(),

    } end
  
return specifications