-- Provides shortcut functions for easier specification readability

-- TODO: Split optionality for resources/spawners

-- local mandatory_for_resources
-- local mandatory_for_spawners

return {
  -- shortcut for using optional in a readable manner
  optional = true,
  -- an integer with exactly one min and one max value
  integer_with_min_max = function (min,max,optional)
    return {
      type = 'integer',
      min  = min,
      max  = max,
      optional = optional or false
      } end,
  -- an integer with a fixed set of possible values
  integer_with_allowed_values = function (allowed_values,optional)
    --convert {'one','two'} to {one=true,two=true} for easier parsing
    local values = {}
    for i=1,#allowed_values do
      values[allowed_values[i] ] = true
      end
    return {
      type = 'integer',
      allowed_values = allowed_values,
      optional = optional or false
      } end,
  decimal_with_min_max = function (min,max,optional)
    return {
      type = 'decimal',
      min  = min,
      max  = max,
      optional = optional or false
      } end,
  -- a string with a fixed set of allowed values
  string_with_allowed_values = function(allowed_values,optional)
    local values = {}
    for i=1,#allowed_values do
      values[allowed_values[i] ] = true
      end
    return {
      type = 'string',
      allowed_values = values,
      optional = optional or false
      } end,
  -- a table containing arbitrary other types
  generic_table = function(values,optional)
    return {
      type = 'generic_table',
      values = values,
      optional = optional or false
      } end,
  -- a standard boolean value
  boolean = function(optional)
    return {
      type = 'boolean',
      optional = optional or false
      } end,
  multi_resource_table = function(count_spec,optional)
    return {
      type = 'multi_resource_table',
      -- min  = min, --min/max is the same for every resource in the table
      -- max  = max,
      count_spec = count_spec,
      optional = optional or false
      } end,
  resource_name = function(optional)
    return {
      type = 'resource_name',
      optional = optional or false
      } end,
  entity_name = function(optional)
    return {
      type = 'entity_name',
      optional = optional or false
      } end,
  unverified = function() --things not yet properly specified
    return {
      type = 'unverified',
      optional = true
      } end, 
  sub_spawns_table = function(values,optional)
    return {
      type = 'sub_spawns_table',
      values = values,
      optional = optional or false
      } end,
  table_or_number = function(values,optional)
    local r = {
      type = 'table_or_number',
      table = values.table,
      number = values.number,
      -- condition = values.condition,
      optional = optional or false
      }
    -- r.values.condition = nil
    return r end,
  }




































  