--[[
    Expected config format:
      local config = { 
      
        'entity-name1' = {
          type = 'resouce-ore',
          richness = 10,
          },
          
        'entity-name2' = {
          type = 'resouce-ore',
          richness = 10,
          },
          
        }
  ]]

local get_validated_entity_config = require 'config-validator/get_validated_entity_config'

local interface_name = 'rso-config-registry'

if remote .remove_interface(interface_name) then
  -- protect against error during later add_interface if name is already in use
  error('RSO can not start, an unknown mod is using the same interface name: '..interface_name)
  end
  
local interface_functions = {}

local validated_configs = {} --will store configs if they're found to be valid

function interface_functions.add_configs (configs)
  if type(configs) ~= 'table' then
    error('RSO config interface must be used with a table argument!')
    end

  for config_name,config in pairs(configs) do
    -- print('\n>>>RSO CONFIG TEST. ENTITY NAME: '..config_name,serpent.line(config))
    local c = get_validated_entity_config(config_name,config)
    --[[ get the configs validated result. result format is:
      local c = {
        valid = LuaBolean,
        config = {resource_name=config_name,richness=10,...}, --the original config with added name if resource
        problems = LuaTable, -- the list of problems that was used to generated the error_message
        error_message = LuaString}
      ]]
    -- print(serpent.block(c))
    if c.valid then
      --Adding a config for an entity is expected to overwrite any
      --previously existing configs for that entity
      validated_configs[config_name] = c.config
      log('RSO succesfully added remotely requested entity config for: '..config_name)
    else
      error(c.error_message)
      end
    end
  return true
  end
  
remote .add_interface(interface_name,interface_functions)
























